import os
import shutil
import pathlib
import openlane
from typing import Literal

def copy_source_folder(source_directory: str, target_directory: str):
    if os.path.exists(target_directory):
        answer = input("Confirm deletion of: " + target_directory)
        if answer.upper() in ["Y", "YES"]:
            shutil.rmtree(target_directory)
        elif answer.upper() in["N", "NO"]:
            print("Copying files now from: " + source_directory + " to " + target_directory)

    shutil.copytree(
        source_directory,
        target_directory,
        symlinks=False,
        ignore=None,
        copy_function=shutil.copy2,
        ignore_dangling_symlinks=False,
        dirs_exist_ok=False,
    )


def setup_example_design(
    project_source: Literal["piel", "openlane"] = "piel",
    example_name:str="simple_design"
):
    """
    We copy the example simple_design from docs to the `/foss/designs` in the `iic-osic-tools` environment.
    """
    if project_source == "piel":
        example_design_folder = os.environ["PIEL_PACKAGE_DIRECTORY"] + "/docs/examples/" + example_name
    elif project_source == "openlane":
        example_design_folder = pathlib.Path(openlane.__file__).parent.resolve() / example_name
    design_folder = os.environ["DESIGNS"] + "/" + example_name
    copy_source_folder(
        source_directory=example_design_folder,
        target_directory=design_folder
    )

def check_example_design(
    example_name:str="simple_design"
):
    """
    We copy the example simple_design from docs to the `/foss/designs` in the `iic-osic-tools` environment.
    """
    design_folder = os.environ["DESIGNS"] + "/" + example_name # TODO verify this copying operation
    return os.path.exists(design_folder)

__all__ = [
    "copy_source_folder",
    "setup_example_design",
    "check_example_design"
]



import json
import openlane
from .defaults import test_spm_open_lane_configuration


def run_openlane_flow(
    configuration: dict | None = test_spm_open_lane_configuration,
    design_directory: str = "/foss/designs/spm",
) -> None:
    """
    Runs the OpenLane flow.

    Args:
        configuration(dict): OpenLane configuration dictionary. If none is present it will default to the config.json file on the design_directory.
        design_directory(str): Design directory PATH.

    Returns:
        None
    """
    if configuration is None:
        # Get extract configuration file from config.json on directory
        config_json_filepath = design_directory + "/config.json"
        config_json = open(config_json_filepath)
        configuration = json.load(config_json)

    Classic = openlane.Flow.get("Classic")

    flow = Classic(
        config=configuration,
        design_dir=design_directory,
    )

    flow.start()


__all__ = ["run_openlane_flow"]



"""
There are a number of ways to generate gdsfactory integration.

It is worth noting that GDSFactory has already the following PDKs installed:
* SKY130nm https://gdsfactory.github.io/skywater130/
* GF180nm https://gdsfactory.github.io/gf180/

"""

import gdsfactory as gf

def create_gdsfactory_component(
    design_directory:str,
) -> gf.Component:
    """
    This function cretes a gdsfactory component that can be included in the network codesign of the device, or that can be used for interconnection codesign.
    """
    pass





"""
The objective of this file is to provide the simulation ports and interconnection to consider modelling digital and mixed signal logic.

The main simulation driver is cocotb, and this generates a set of files that correspond to time-domain digital simulations.
The cocotb verification software can also be used to perform mixed signal simulation, and digital data can be inputted as a bitstream into a photonic solver, although the ideal situation would be to have integrated photonic time-domain models alongside the electronic simulation solver, and maybe this is where it will go. It can be assumed that, as is currently, cocotb can interface python with multiple solvers until someone (and I'd love to do this) writes an equivalent python-based or C++ based python time-domain simulation solver.

The nice thing about cocotb is that as long as the photonic simulations can be writen asyncrhonously, time-domain simulations can be closesly integrated or simulated through this verification software.
"""
from typing import Literal


def configure_cocotb_simulation(
    design_directory: str,
    simulator: Literal["icarus", "verilator"],
    top_level_language: Literal["verilog", "vhdl"],
    top_level_verilog_module: str,
    test_python_module: str,
    verilog_sources: list,
):
    """
    Writes a cocotb makefile

    In the form:
         Makefile
        # defaults
        SIM ?= icarus
        TOPLEVEL_LANG ?= verilog

        VERILOG_SOURCES += $(PWD)/my_design.sv
        # use VHDL_SOURCES for VHDL files

        # TOPLEVEL is the name of the toplevel module in your Verilog or VHDL file
        TOPLEVEL = my_design

        # MODULE is the basename of the Python test file
        MODULE = test_my_design

        # include cocotb's make rules to take care of the simulator setup
        include $(shell cocotb-config --makefiles)/Makefile.sim
    """
    pass


def run_cocotb_simulation(
    design_directory: str,
):
    """
    Equivalent to running the cocotb makefile.
    """
    # make
    pass


write_cocotb_makefile = configure_cocotb_simulation
make_cocotb = run_cocotb_simulation


__all__ = [
    "configure_cocotb_simulation",
    "make_cocotb",
    "run_cocotb_simulation",
    "write_cocotb_makefile",
]
